%===================================================================================
%
% A PDE discretizer for Matlab, restricted to a uniform mesh on the unit square
% [0,1] x [0,1] in 2D space. This sets up in COO sparse matrix format the
% coefficient matrix A coming from the five point stencil operator on the square
% domain. The data structure used is three arrays of length nz = number of nonzeros
% in the matrix A (which is also the number of unknowns in the PDE mesh). Those 
% arrays are
%   rows(i) = row number of the i-th nonzero value
%   cols(i) = col number of the i-th nonzero value
%   vals(i) = the value  of the i-th nonzero value
% In most programming languages, rows and cols are integer arrays while vals is an
% array of doubles. [Matlab doesn't do integers.] At the same time this will create
% the right-hand side vector g for the linear system A*v = g. 
%
% N is the number of unknowns in each direction on a square domain, so
% with given Dirichlet values on the boundaries, the discretized physical
% domain is (N+2)x(N+2). The distance between two adjacent nodes in a row or col of
% the mesh is h = 1/(N+1). If the numbering of nodes starts with 0, then
%     for x, ix = 0   corresponds to an x-value of 0.0
%     for x, ix = N+1 corresponds to an x-value of 1.0
%     for y, iy = 0   corresponds to an y-value of 0.0
%     for y, iy = N+1 corresponds to an y-value of 1.0
% In those cases the PDE function u(x,y) have values given by BC(x,y), and so are 
% not unknowns. The number of unknowns is n = N*N. 
%
% The unknown function u(x,y) will have its values at the nodes strung out along
% the 1D array v, so it is necessary to impose some numbering scheme. This uses
% the so-called "natural order", which orders the nodes from bottom to top and
% left to right in the mesh of unknowns. So v(1:N) are the points corresponding
% u(x,h), v(N+1:2N) correspond to u(x,2h), ... etc.
%
% This routine calls three functions, in separate files: 
%   BC.m, which defines the boundary conditions, and
%   getsten.m, which for a given value of x, y, and h gives the five values in 
%       the coefficient matrix for the corresponding row of A, and the value of
%       the corresponding element in the right-hand side vector.
%   directions.m, which gives names to the indices in stencil().
%
% getsten() in turn calls the coefficient functions in the functions afun.m,
%       bfun.m, ... ffun.m. 
%
%---------------
% Randall Bramley
% Department of Computer Science
% Indiana University
%--------------------
% Last Modified: Thu 05 Apr 2018, 02:00 PM
%===================================================================================
function [rows, cols, vals, g] = setup(N)

    node    = 1;  % the current row number of the matrix 
    nzcount = 1;  % how many nonzeros in the matrix have been created so far
    nxoff   = 1;  % the offset between adjacent nodes in the horizontal direction
    nyoff   = N;  % the offset between adjacent nodes in the vertical direction

    n = N*N;  % Total number of unknowns

    %-----------------------------------------------------------------------------
    % Preallocate space for arrays. Matlab will automatically extend an array
    % if an index exceeds its size on a write, but that can involve large amounts
    % of memory allocation/deallocation. The formula for the number of nonzeros
    % used here is 5*N^2   -   4*N  
    %               ^           ^ 
    %               |           | 
    %  each node has 5 nonzeros |
    %                           |
    %    except for nodes bordering a BC
    %
    % This subtracts off a nonzero from the four corners twice, which is correct 
    % since their 5-pt stencil entails 2 boundary nodes, not just one.
    %-----------------------------------------------------------------------------
    totalnnz = 5*N^2 - 4*N;

    % Coefficient matrix data structure
    vals = zeros(totalnnz, 1);
    cols = zeros(totalnnz, 1);
    rows = zeros(totalnnz, 1);
    % Right hand side vector; initialized to NaN to catch any mistakes
    g = NaN*zeros(n, 1);

    % Mesh spacing size
    h = 1/(N+1);
    % Get the directions for the five-pt centered diffs
    [center,left,down,right,up,rhs] = directions();

    % Loop over mesh points, x varying fastest:
    for iy = 1:N 
        y = iy*h;  % corresponding y-coordinate

        for ix = 1:N 
            x = ix*h;  % corresponding x-coordinate

            stencil = getsten(x, y, h);

            g(node) = stencil(rhs);

            % Away from bottom boundary?
            if (iy > 1)              
                rows(nzcount) = node;
                cols(nzcount) = node - nyoff;
                vals(nzcount) = stencil(down);
                nzcount = nzcount + 1;
            else
                g(node) = g(node) - stencil(down)*BC(x, 0.0);
            end

            % Away from left boundary?
            if (ix > 1)
                rows(nzcount) = node;
                cols(nzcount) = node - nxoff;
                vals(nzcount) = stencil(left);
                nzcount = nzcount + 1;
            else
                g(node) = g(node) - stencil(left)*BC(0.0, y);
            end

            % Central node in stencil = diagonal element of A
            rows(nzcount) = node;
            cols(nzcount) = node;
            vals(nzcount) = stencil(center);
            nzcount = nzcount + 1;

            % Away from right boundary?
            if (ix < N)
                rows(nzcount) = node;
                cols(nzcount) = node + nxoff;
                vals(nzcount) = stencil(right);
                nzcount = nzcount + 1;
            else
                g(node) = g(node) - stencil(right)*BC(1.0, y);
            end

            % Away from top boundary?
            if (iy < N)
                rows(nzcount) = node;
                cols(nzcount) = node + nyoff;
                vals(nzcount) = stencil(up);
                nzcount  = nzcount + 1;
            else
                g(node) = g(node) - stencil(up)*BC(x, 1.0);
            end

            % Increment variable counter:
            node = node + 1;

        end  % ix loop
    end  % iy loop

    % Total number of unknowns:
    n = node - 1;
    if (n ~= N*N)
        disp(sprintf('Problem in setup; n = %d but N^2 = %d', n, N*N))
        error('Bailing out; fix this mess.')
    end

    % Total number of nonzeros in coefficient matrix:
    nz = nzcount - 1;
    % ... and verify it just like with the number of unknowns, as a secondary check.
    if (totalnnz ~= nz)
        disp(sprintf('Problem in setup; nz = %d but totalnnz = %d', ...
            nz, totalnnz))
        error('Bailing out; fix this mess.')
    end

return
