%------------------------------------------------------------------------------------
% Script to set up five point difference matrices and display them. The script
% sampleSparsity.m shows just the nonzero sparsity structure of the matrix (via
% Matlab's spy() function), but this tries to also show the values in the array. It
% can use the log of the array entries to handle a wide range of magnitudes of
% values. It is best to display the absolute values of the entries, but that can be
% turned off via the Boolean absolute.
%
% This takes the COO data structure from setup.m and converts it to a full 
% matrix. so only do this for small values of N. A safety check will barf and return
% if n > 2000.
%
% Boolean variables named doxxx control what plotting methods to use. The
% only worthwhile one is probably dopcolors.
%
%--------------
% Randall Bramley
% Department of Computer Science
% Indiana University, Bloomington
%-----------------
% Started: Wed 09 Nov 2016, 05:04 PM
% Last Modified: Tue 03 Apr 2018, 03:23 PM
%------------------------------------------------------------------------------------

%======================================================================
% User modifiable booleans to turn stuff on/off. Recommended defaults:
%   dopcolors, logarithmic, and absolute = true and all others false.
%======================================================================

    % Control the plotting methods to use ..  
    domeshes    = false;
    doplot2     = false; % probably the least useful visualization
    doplot3     = false; % best with absolute = true 
    dosurfs     = false;
    dopcolors   = true; 
    % Display absolute values or logs thereof  
    absolute    = true;
    logarithmic = true;
    % Whether or not to ask user for problem size
    userinput   = true; 

%=======================================================
% End of user modifiable booleans to turn stuff on/off  
%=======================================================

    if userinput
        N = input('Number of unknowns in each direction [default 8]: ');
        disp(' ')
    else
        N = 8;
    end
    if isempty(N)
        N = 8;
    end

    n = N^2; 
    disp(['Total number of unknowns is ', num2str(n)])
    if (n > 2000)
        disp('vizmatrix will create a dense n x n array and display it.')
        disp('Even with a 4k screen, you are not likely to see much with')
        disp('that large of a matrix. Set N smaller, or change the upper')
        disp('bound in vizmatrix to be larger.')
        return
    end


    [rows, cols, vals, b] = setup(N);
    S = sparse(rows, cols, vals);
    A = full(S);

    % Can use the next line to limit range, e.g. 
    %   lowerlimit = 4; upperlimit = 10;
    lowerlimit = 1; upperlimit = n;
    r = lowerlimit:upperlimit;
    c = lowerlimit:upperlimit;
    
    % Matlab idiomatic way to set up for mesh(), surf(), etc.
    [R, C] = meshgrid(lowerlimit:upperlimit, lowerlimit:upperlimit);
    G = A;
    if (absolute) 
        G = abs(G);
    else
        if (logarithmic)
            G = log(abs(G));
        end
    end

    if domeshes
        positionfig;
        mesh(R, C, G(r, c));
        shading interp;
        title('mesh view');
    end

    if doplot3
        positionfig;
        if (absolute)
            plot3(rows, cols, abs(vals), 'b*');
            hold on
            lh = line([rows';rows'], [cols';cols'], [zeros(size(vals))'; abs(vals)'], ...
                'Color', 'k');
            hold off
        else
            plot3(rows, cols, vals, 'b*');
        end
        axis ij
        axis square
        grid on;
    end

    if doplot2
        positionfig;
        % May want to also try
        %   [c, h] = contourf(R, C, G);
        [c, h] = contour(R, C, G);
        % Put into matrix coordinates, and give square aspect ratio 
        axis ij
        axis square
        colorbar;
        % clabel(c, h);
    end

    if dosurfs
        positionfig;
        surf(R, C, G(r, c));
        % Use surfc(R, C, G(r, c)) instead to have contours also drawn
        shading interp;
        title('surf view');
        colorbar;
    end

    % Closest approximation to what emily does. However, zooming in is not as useful.
    if dopcolors
        % Drawing border lines gives an all-black figure for large n
        positionfig;
        pcolor(G(r, c));
        if n < 401
            shading faceted; % default shading is faceted, anyway  
            title('pcolor faceted');
        else
            shading flat;
            title('pcolor flat');
        end
        
        % Turn the zero values to be black in colormap  
        c = colormap;
        lastcolors = 1;
        c(1:lastcolors, :) = 0;
        colormap(c);
        colorbar;
       
        % Put into matrix coordinates, and give square aspect ratio 
        axis ij
        axis square
    end

